<?php
/*
Plugin Name: Shibuya CMS
Plugin URI: http://heli.otro.pics/shibuya-cms/
Description: カスタム投稿タイプを簡単に登録でき、カスタムフィールドと合わせた投稿画面が作れます。
Author: Takuma Ando
Version: 0.2.2
Author URI: http://heli.otro.pics/
*/


define( 'SHIBUYA_CMS_DIR', dirname(__FILE__) . '/post-types/');

function shibuya_cms_files() {
	static $files;
	if( empty($files) ) {
		$files = array();
		foreach( scandir(SHIBUYA_CMS_DIR) as $file ) {
			if( $file != '.' && $file != '..' && substr($file, 0, 1) != '_' ) {
				$files[] = $file;
			}
		}
	}
	return $files;
}


add_action( 'init', 'shibuya_cms_init' );
function shibuya_cms_init() {
	global $wp_post_types, $wp_taxonomies;
	
	foreach( shibuya_cms_files() as $file ){
		$post_type = str_replace( array( 'post-type-', '.php' ), '', $file );
		$data = file_get_contents(dirname(__FILE__) . "/post-types/$file");
		$params = array(
			'label'              => $post_type,
			'public'             => true, 
			'query_var'          => true,
			'rewrite'            => array(
				'slug'         => $post_type,
			),
			'capability_type'    => 'post',
			'hierarchical'       => false,
			'menu_position'      => 5,
			'supports'           => 'title,editor',
			'has_archive'        => true,
		);
		$taxonomies = $metas = array();
		
		if( preg_match( '/^.*?<\?php.*?\/\*(.*?)\*\//is', $data, $matches ) ){
			$header = trim( $matches[1] );
			if( preg_match_all( '/([^\:\n]*)[\s]*?\:[\s]*?([^\:\n]*)/im', $header, $matches ) ){
				foreach( $matches[1] as $i => $key ){
					$key   = trim( $key );
					$value = trim( $matches[2][$i] );
					
					//true or false
					if( ! strcasecmp($value, 'true') )
						$value = true;
					if( ! strcasecmp($value, 'false') )
						$value = false;
					
					//tax-とcat-はカテゴリ
					if ( preg_match('/^(?:tax|cat)\-(.*)$/i', $key, $_matches ) ) {
						$taxonomies[ $_matches[1] ] = array( $value, 'category' );
					}
					//tag-はタグ
					elseif ( preg_match('/^tag\-(.*)$/i', $key, $_matches ) ) {
						$taxonomies[ $_matches[1] ] = array( $value, 'tag' );
					}
					//リストに出力する項目
					elseif ( preg_match('/^list\-meta\-(.*)$/i', $key, $_matches ) ) {
						$metas[ $_matches[1] ] = $value;
					}
					//その他のパラメータ
					else {
						$key = strtolower( $key );
						if( strstr( $key, '.' ) ) {
							$key = explode('.', strtolower($key));
							if( !isset( $params[ $key[0] ] ) )
								$params[ $key[0] ] = (object) array();
							$params[ $key[0] ]->{$key[1]} = is_numeric( $value )
																			? $value - 0
																			: $value;
						} else {
							$params[ $key ] = is_numeric( $value )
																			? $value - 0
																			: $value;
						}
					}
				}
			}
		}
		
		if( file_exists( SHIBUYA_CMS_DIR . "/post-types/icon-{$post_type}.png" ) ){
			$params[ 'menu_icon' ] = plugin_dir_url(__FILE__) . "post-types/icon-{$post_type}.png";
		}
		
		if( $post_type != 'post' && $post_type != 'page' ) {
			if( ! post_type_exists( $post_type ) ) {
				foreach( $taxonomies as $key => $value ){
					if( !get_taxonomies( array( 'name'=>$key ) ) ){
						register_taxonomy( $key, $post_type, array(
							'label'        => $value[0],
							'public'       => true,
							'show_ui'      => true,
							'hierarchical' => $value[1] == 'category' ? true : false,
							'rewrite'      => array(
								'slug'         => $key,
							),
						) );
					}
				}
				if( $taxonomies && !isset( $params['taxonomies'] ) ){
					$params['taxonomies'] = array_keys( $taxonomies );
				}
				
				if( $params['supports'] )
					$params['supports'] = explode( ',', $params['supports'] );
				
				register_post_type( $post_type, $params );
			}
		} else {
			$post_type_object = &$wp_post_types[ $post_type ];
			foreach( $params as $key => $value ){
				if( $key != 'labels' ) {
					$post_type_object->{$key} = $value;
				} else {
					foreach( $value as $_key => $_value ) {
						$post_type_object->labels->{$_key} = $_value;
					}
				}
			}
			$post_type_object->taxonomies = $taxonomies;
			if( $post_type == 'post' ) {
				foreach( array('category', 'post_tag') as $taxonomy ) {
					if( !isset( $taxonomies[$taxonomy] ) 
									&& count( $wp_taxonomies[$taxonomy]->object_type ) == 1
									&& $wp_taxonomies[$taxonomy]->object_type[0] == $post_type ) {
						unset( $wp_taxonomies[$taxonomy] );
					}
				}
			}
		}
		
	}
	
	add_action( 'admin_menu', 'shibuya_cms_admin_menu_fix' );
}



add_action( 'admin_init', 'shibuya_cms_admin_init' );
function shibuya_cms_admin_init() {
	if( !empty( $_POST['_shibuya_cms_add_post_type'] ) ) {
		$bool = !empty( $_POST['post_type_slug'] )
							&& !empty( $_POST['post_type_name'] );
		$bool = $bool && preg_match( '/^[a-zA-Z0-9\_\-]+$/', $_POST['post_type_slug'] );
		$bool = $bool && !file_exists( SHIBUYA_CMS_DIR . $_POST['post_type_slug'] . '.php' );
		
		if( $bool ) {
			$data = file_get_contents( dirname(__FILE__) . '/post-type-sample.php' );
			$data = preg_replace( '/\{\{post_type_name\}\}/', $_POST['post_type_name'], $data );
			@file_put_contents( SHIBUYA_CMS_DIR .  $_POST['post_type_slug'] . '.php', $data );
			if( function_exists( 'chmod' ) )
				@chmod( SHIBUYA_CMS_DIR .  $_POST['post_type_slug'] . '.php', 0666 );
			wp_redirect($_SERVER['REQUEST_URI'], 302);
			exit;
		}
	}
	
	add_action( 'admin_enqueue_scripts', 'shibuya_cms_enqueue_script' );
}


function shibuya_cms_enqueue_script(){
	wp_enqueue_style( 'shibuya-cms',   plugin_dir_url(__FILE__) . 'shibuya-cms.css' );
	wp_enqueue_style( 'jquery-ui-theme',   plugin_dir_url(__FILE__) . 'jquery-ui-theme/jquery-ui.min.css' );
	wp_enqueue_script( 'shibuya-cms', plugin_dir_url(__FILE__) . 'shibuya-cms.js', array(
					'jquery',
					'jquery-ui-core',
					'jquery-ui-datepicker',
					'jquery-ui-sortable',
					'jquery-ui-draggable',
					'jquery-ui-droppable',
					'jquery-ui-selectable' ) );
	echo '<script>var shibuyaCmsCustomFields; var SHIBUYA_CMS_DIR = \''. plugin_dir_url(__FILE__) . '/shibuya-cms\'; var SHIBUYA_CMS_IMAGE_URL = \'' . admin_url() . '?shibuya-cms-attachment.php\';</script>';
}



add_action( 'admin_menu', 'shibuya_cms_admin_menu' );
function shibuya_cms_admin_menu() {
	add_options_page( 'Shibuya CMS', 'Shibuya CMS', 'manage_options', 'shibuya_cms_plugin_page', 'shibuya_cms_plugin_page');
}

function shibuya_cms_plugin_page() { ?>

<h2>Shibuya CMS</h2>

<p>カスタム投稿タイプの設定ファイルをつくり、そのファイルに記載した内容で、投稿画面を作成できます。</p>
<p>カスタム投稿の設定はテーマやプラグインと同じくファイル冒頭のコメントアウトを利用します。</p>

<div style="border:1px solid #ccc; background:white; margin-bottom:20px;">
	<p style="margin:0; padding:2px 8px 1px; border-bottom:1px solid #ccc;">投稿タイプ設定ファイル一覧</p>
	<?php if( shibuya_cms_files() ) : ?>
	<ul style="margin:0; padding:8px">
	<?php foreach( shibuya_cms_files() as $file ) : if($file == '.' || $file == '..') continue; ?>
		<li><a href="<?php echo admin_url('/plugin-editor.php?' . http_build_query(array(
									'file'   => "shibuya-cms/post-types/{$file}",
									'plugin' => 'shibuya-cms/shibuya-cms.php',
									)) ) ?>" target="_blank"><?php echo $file ?></a></li>
	<?php endforeach ?>
	</ul>
	<?php else : ?>
	<p style="margin:0; padding:8px">まだ投稿タイプ設定ファイルはありません</p>
	<?php endif ?>
</div>


<form method="post" action="<?php echo $_SERVER['REQUEST_URI'] ?>" novalidate="novalidate" style="border-top: 1px solid #ccc;">
	<input type="hidden" name="_shibuya_cms_add_post_type" value="1" />
	<p>
		<label>SLUG : <input type="text" name="post_type_slug" /></label>　
		<label>NAME : <input type="text" name="post_type_name" /></label>　
		<input type="submit" name="submit" id="submit" class="button button-primary" value="追加">
	</p>
</form>

<?php }





add_action( 'add_meta_boxes', 'shibuya_cms_add_meta_boxes' );
function shibuya_cms_add_meta_boxes(){
	global $post;
	$post_type = !empty( $_GET['post_type'] ) ? $_GET['post_type'] : '';
	if( !empty( $post ) ){
		$post_type = $post->post_type;
	}
	if( post_type_exists( $post_type ) && file_exists( SHIBUYA_CMS_DIR .  "{$post_type}.php" ) ){
		$code = file_get_contents( SHIBUYA_CMS_DIR . "{$post_type}.php" );
		if( trim( preg_replace( '/\/\*[\s\S]*?\*\//sm', '', $code ) ) != '<?php' ){
			$post_type = get_post_type_object( $post_type );
			add_meta_box(
				$post_type->name.'-data',
				$post_type->label,
				'shibuya_cms_meta_box_inner',
				$post_type->name,
				'normal',
				'high',
				array( $post_type )
			);
		}
	}
}

function shibuya_cms_meta_box_inner( $post, $meta_box ){
	$post_type = $meta_box['args'][0]->name;
	if( post_type_exists( $post_type ) ){
		echo '<div class="shibuya-cms-form">';
		if( $custom_fields = get_post_custom( $post->ID ) )
			echo '<script>shibuyaCmsCustomFields = '.json_encode( $custom_fields ) . '</script>';
		include SHIBUYA_CMS_DIR .  "{$post_type}.php";
		echo '</div>';
	}
}

add_action( 'save_post', 'shibuya_cms_save_post' );
function shibuya_cms_save_post( $post_id ){
	if( !empty( $_POST ) ){
		$postdata = array();
		foreach( $_POST as $key => $value ){
			if( substr( $key, 0, 1 ) == '@' ){
				$key = substr( $key, 1 );
				if( ! is_array( $value ) ) {
					update_post_meta( $post_id, $key, $value );
				} else {
					delete_post_meta( $post_id, $key );
					foreach( $value as $v ){
						add_post_meta( $post_id, $key, $v );
					}
				}
			}
		}
	}
}



function shibuya_cms_manage_posts_columns( $columns ){
	if( !empty( $GLOBALS['shown-metas'] ) ){
		foreach( $GLOBALS['shown-metas'] as $key => $value ){
			$columns[ $key ] = $value;
		}
		return $columns;
	}
}
function shibuya_cms_manage_posts_custom_column( $column_name, $post_id ){
	if( isset( $GLOBALS['shown-metas'][ $column_name ] ) ){
		if( $values = get_post_meta( $post_id, $column_name ) )
			echo join( $values, ', ');
		else
			echo __('None');
	}
}

/**
 * 投稿の画像を合わせるための処理
 */
add_action('init', 'shibuya_cms_get_post_image');
function shibuya_cms_get_post_image(){
	$url = ( !empty($_SERVER['HTTPS']) ? 'https://' : 'http://' )
				. $_SERVER['HTTP_HOST']
				. $_SERVER['REQUEST_URI'];
	if( is_user_logged_in() && strstr( $url, admin_url() . '?shibuya-cms-attachment.php' ) ){
		//idsの場合はjsonで複数のURL、idの場合は画像のURLを返す
		if( isset($_GET['ids']) ) {
			header('Content-Type: application/json');
			$result = array();
			foreach( explode(',', $_GET['ids']) as $value ) {
				$result[$value] = wp_get_attachment_thumb_url( $value );
			}
			$result['dummy'] = plugin_dir_url(__FILE__) . 'img/dummy.png';
			echo json_encode($result);
		} else {
			header('Content-Type: text/plain');
			if( !empty( $_GET['id'] ) ) {
				$data = wp_get_attachment_image_src( $_GET['id'], 'thumbnail' );
				if( !empty( $data[0] ) ) {
					echo $data[0];
					exit;
				}
			}
			echo plugin_dir_url(__FILE__) . 'img/dummy.png';
		}
		exit;
	}
}



function shibuya_cms_admin_menu_fix() {
	global $wp_post_types, $wp_taxonomies, $menu, $submenu;
	$links = array();
	
	if( file_exists( dirname(__FILE__) . '/post-types/post.php' ) ) {
		$links['edit.php']     = $wp_post_types['post']->labels->menu_name;
		$links['post-new.php'] = $wp_post_types['post']->labels->add_new;
		if( empty( $wp_post_types['post']->taxonomies['category'] ) )
			$links['edit-tags.php?taxonomy=category'] = null;
		if( empty( $wp_post_types['post']->taxonomies['post_tag'] ) )
			$links['edit-tags.php?taxonomy=post_tag'] = null;
	}
	if( file_exists( dirname(__FILE__) . '/post-types/page.php' ) ) {
		$links['edit.php?post_type=page']     = $wp_post_types['page']->labels->menu_name;
		$links['post-new.php?post_type=page'] = $wp_post_types['page']->labels->add_new;
	}
	
	foreach( $links as $link => $label ) {
		foreach( $menu as $idx => $menu_arr ){
			if( $menu_arr[2] == $link )
				$menu[$idx][0] = $label;
		}
		foreach( $submenu as $menu_link => $submenu_items ) {
			foreach( $submenu_items as $idx => $item ) {
				if( $item[2] == $link ) {
					if( $label )
						$submenu[$menu_link][$idx][0] = $label;
					else
						unset( $submenu[$menu_link][$idx][0] );
				}
			}
		}
	}
}

/**
 * 編集画面でWYSIWYGエディタを増やす
 * 投稿タイプ設定ファイル内で shibuyaCmsTinyMCESettings
 */
function shibuya_cms_tinymce_settings( $settings ) {
	$init = $settings;
	unset( $init['formats'], $init['external_plugins'] );
	echo '<script>shibuyaCmsTinyMCESettings = jQuery.extend({ formats:' . $settings['formats'] . ', external_plugins:' . (
				$settings['external_plugins'] ? $settings['external_plugins'] : '{}'
				) . '}, ' . json_encode( $init ) . ');</script>' . PHP_EOL;
	return $settings; 
}
add_filter('tiny_mce_before_init','shibuya_cms_tinymce_settings');